<?php
/**
 * LDAP Access
 *
 * --
 * Alain Reguera Delgado <alain.reguera@gmail.com>
 ***/

class LDAP
{
    public $ldap_host;
    public $ldap_port;
    public $ldap_basedn;
    public $ldap_rootdn;
    public $ldap_rootpw;
    public $ldap_authschema;
    public $ldap_conn;
    public $ldap_bind;


   /*** 
    * Class initialization
    * ----------------------------------------------------
    ***/

    function __construct()
    {
        // Initialize LDAP configuration
        $this->ldap_host        = LDAP_HOST;
        $this->ldap_port        = LDAP_PORT;
        $this->ldap_basedn      = LDAP_BASEDN;
        $this->ldap_rootdn      = LDAP_ROOTDN;
        $this->ldap_rootpw      = LDAP_ROOTPW;
        $this->ldap_authschema  = LDAP_AUTHSCHEMA;

        // Open connection against ldap server
        $this->ldap_conn = ldap_connect( $this->ldap_host, $this->ldap_port) or die( ldap_error( $this->ldap_conn ) );

        // Set protocol version to use LDAPv3 
        ldap_set_option($this->ldap_conn, LDAP_OPT_PROTOCOL_VERSION, 3) or die( ldap_error( $this->ldap_conn ) );

        // Bind
        if ( $this->ldap_rootpw != '' )
        {
            $this->ldap_bind = ldap_bind( $this->ldap_conn, $this->ldap_rootdn, $this->ldap_rootpw ) or die( $this->ldap_conn ); 
        }
    }

   /*** 
    * Verify LDAP uid's value uniqness
    * ----------------------------------------------------
    ***/

    function is_uid_present( $uid )
    {
        $filter     = 'uid=' . $uid;
        $result     = ldap_search( $this->ldap_conn, $this->ldap_basedn, $filter);
        $entry      = ldap_get_entries( $this->ldap_conn, $result);

        if ( $uid != '' && $entry['count'] == 1 )
        {
            return true;
        }
        else
        {
            return false;
        }
    }

   /*** 
    * Prepare LDAP userPassword attribute
    * ----------------------------------------------------
    ***/

    function prepare_userpassword( $userpassword )
    {
        $dirty['userpassword'] = $userpassword;

        switch ( $this->ldap_authschema )
        {
            case '{MD5}':
            $clean['userpassword'] = '{MD5}' . base64_encode( pack( 'H*', md5( $dirty['userpassword'] ) ) );
            break;
    
            case '{SHA}':
            $clean['userpassword'] = '{SHA}' . base64_encode( pack( 'H*', sha1( $dirty['userpassword'] ) ) );
            break;
        }

        return $clean['userpassword'];
    }


   /*** 
    * Add User
    * ----------------------------------------------------
    ***/

    function add_User( $entry )
    {
        $ldap_entry = array();

        // Define user DN
        $dn = 'uid=' . $entry['email'] . ',' . $this->ldap_basedn;
                
        // Remove user if exists
        if ( $this->is_uid_present( $entry['uname'] ) === true )
        {
            $this->delete_User( $entry );
        }

        // Prepare userPassword and other attributes for insertion in LDAP directory.
        $ldap_entry['objectclass']  = 'inetOrgPerson';
        $ldap_entry['cn']           = $entry['name'];
        $ldap_entry['mail']         = $entry['email'];
        $ldap_entry['userpassword'] = $this->prepare_userpassword($entry['pass']);
        $ldap_entry['sn']           = preg_replace('/^([a-zA-Z0-9_]+ ?)/','', $ldap_entry['cn']);
        $ldap_entry['uid'][0]       = $ldap_entry['mail'];
        $ldap_entry['uid'][1]       = $entry['uname'];
        $ldap_entry['displayname']  = $entry['uname'];
        $ldap_entry['employeetype'] = 'writer';
        $ldap_entry['preferredlanguage'] = 'en';

        if (ldap_add( $this->ldap_conn, $dn, $ldap_entry ))
        {
            return true; 
        } 
        else
        {
            return false;
        }
    }


   /*** 
    * Delete User 
    * ----------------------------------------------------
    ***/

    function delete_User( $entry )
    {
        // Define user DN
        $dn = 'uid=' . $entry['email'] . ',' . $this->ldap_basedn;

        if ( ldap_delete( $this->ldap_conn, $dn ) ) 
        {
            return true;
        }
        else
        {
            return false;
        }
    }

   /*** 
    * Update User 
    * ----------------------------------------------------
    * Update LDAP userPassword only.
    ***/

    function update_userPassword( $dn, $userPassword )
    {
        $entry = array('userpassword' => $userPassword ); 

        if ( ldap_modify( $this->ldap_conn, $dn, $entry) ) 
        {
            return true; 
        }
        else
        {
            return false; 
        }
    }

   /*** 
    * Send email notification 
    * ----------------------------------------------------
    * Send notification by email.
    ***/

    function send_emailNotification( $To, $Body )
    {
        if ( MAIL_NOTIFICATION == 'YES' )
        {
            $Subject = 'CentOS Forum Password Notification.';
            if ( mail( $To, $Subject, $Body ) === true )
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false; 
        }
    }

   /*** 
    * Get LDAP user list 
    * ----------------------------------------------------
    * 1. Show a form with a list of all users inserted from xoops.users table.
    * 2.  Generate random passwords for each user and codify them into
    * userPassword format. 3. Real passwords are not displayed.
    ***/

    function get_userList()
    {
        global $newbb_to_phpbb;

        // Get users from LDAP server
        $filter = 'objectclass=inetorgperson';
        $result = ldap_search( $this->ldap_conn, $this->ldap_basedn, $filter);
        $users = ldap_get_entries( $this->ldap_conn, $result );
        
        $html = '<p>Total user(s): ' . $users['count'] . '</p>';
        $html .= '<table border="1">';
        $html .= '<tr>';
        $html .= '<th>DN</th>';
        $html .= '<th>CN</th>';
        $html .= '<th>NewPass</th>';
        $html .= '<th>userPassword</th>';
        $html .= '<th>Password Updated</th>';
        $html .= '<th>Email Notification</th>';
        $html .= '</tr>';

        for ($i = 0; $i < $users['count']; $i++)
        {
            // Reset userPassword value in a random manner
            $newPassword = $newbb_to_phpbb->get_randomPass();
            $userPassword = $this->prepare_userpassword($newPassword);

            $html .= '<tr>';
            $html .= '<td>' . $users[$i]['dn'] . '</td>'; 
            $html .= '<td>' . $users[$i]['cn'][0] . '</td>'; 
            $html .= '<td>' . $newPassword . '</td>'; 
            $html .= '<td>' . $userPassword . '</td>'; 

            // Update LDAP userPassword field
            if ( $this->update_userPassword( $users[$i]['dn'], $userPassword ) === true )
            {
                $html .= '<td class="center">YES</td>'; 
            }
            else
            {
                $html .= '<td class="center">NO</td>'; 
            }
            
            // Send email notification
            $To   = $users[$i]['mail'][0];
            $Body = 'Hi ' . $users[$i]['cn'][0] . ',' . "\n\n";
            $Body .= 'You new password is: ' . $newPassword . "\n\n";
            if ( $this->send_emailNotification( $To, $Body ) === true )
            {
                $html .= '<td class="center">SENT</td>'; 
            }
            else
            {
                $html .= '<td class="center">NOT SENT</td>'; 
            }
            $html .= '</tr>';
        }
         
        $html .= '</table>';

        return $html;
    }

   /*** 
    * Class destruct
    * ----------------------------------------------------
    ***/

    function __destruct()
    {
        if ( isset( $this->ldap_conn ) ) 
        {
            ldap_unbind( $this->ldap_conn );
        }
    }
}

$ldap = new LDAP;
?>
